/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
package vt.cs.widgets;

import java.awt.*;

/**
 * RadarPaneLayout - This Layout is used by RadarPane
 *
 * @version 	%I%, %G%
 * @author 	James Begole
 */

class RadarPaneLayout implements LayoutManager, java.io.Serializable {
  int hgap = 0;
  int vgap = 0;

  RadarPaneClientPanel clientPanel;
  Scrollbar horizScroll;
  Scrollbar vertScroll;
  Component radarToggle;
 
    /**
     * Constructs a new RadarPaneLayout.
     */
     RadarPaneLayout() {
    }

    /**
     * Constructs a RadarPaneLayout with the specified gaps.
     * @param hgap the horizontal gap
     * @param vgap the vertical gap
     */
     RadarPaneLayout(int hgap, int vgap) {
	this.hgap = hgap;
	this.vgap = vgap;
    }

    /**
     * Adds the specified named component to the layout.
     * @param name the String name
     * @param comp the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
      debug("addLayoutComponent");
	if ("Client".equals(name)) {
	    clientPanel = (RadarPaneClientPanel)comp;
	} else if ("Horizontal".equals(name)) {
	    horizScroll = (Scrollbar)comp;
	} else if ("Vertical".equals(name)) {
	    vertScroll = (Scrollbar)comp;
	} else if ("RadarToggle".equals(name)) {
	    radarToggle = comp;
	}
    }


    /**
     * Removes the specified component from the layout.
     * @param comp the component to be removed
     */
    public void removeLayoutComponent(Component comp) {
      if (comp == clientPanel) {
	clientPanel = null;
      } else if (comp == horizScroll) {
	horizScroll = null;
      } else if (comp == vertScroll) {
	vertScroll = null;
      }
    }


    /**
     * Returns the minimum dimensions needed to layout the components
     * contained in the specified target container. 
     * @param target the Container on which to do the layout
     * @see Container
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container target) {
      debug("minimumLayoutSize");
	Dimension dim = new Dimension(0, 0);

	if ((clientPanel != null) && clientPanel.isVisible()) {
	  Dimension d = clientPanel.getMinimumSize();
	  dim.width += d.width + hgap;
	  dim.height = d.height + vgap;
	  //	  resizeAdjustables(d, d);
	}

	if ((horizScroll != null) && horizScroll.isVisible()) {
	  Dimension d = horizScroll.getMinimumSize();
	  dim.width =  Math.max(d.width, dim.width);
	  dim.height += d.height + vgap;
	}

	if ((vertScroll != null) && vertScroll.isVisible()) {
	  Dimension d = vertScroll.getMinimumSize();
	  dim.width += d.width + hgap;
	  dim.height = Math.max(d.height, dim.height);
	}
	

	Insets insets = target.getInsets();
	dim.width += insets.left + insets.right;
	dim.height += insets.top + insets.bottom;

	return dim;
    }


    
    /**
     * Returns the preferred dimensions for this layout given the components
     * in the specified target container.
     * @param target the component which needs to be laid out
     * @see Container
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container target) {
      debug("preferredLayoutSize");
	Dimension dim = new Dimension(0, 0);

	if ((clientPanel != null) && clientPanel.isVisible()) {
	  Dimension d = clientPanel.getPreferredSize();
	  dim.width += d.width + hgap;
	  dim.height = d.height + vgap;
	  //	  resizeAdjustables(d, d);
	}
	if ((horizScroll != null) && horizScroll.isVisible()) {
	  Dimension d = horizScroll.getPreferredSize();
	  dim.width =  Math.max(d.width, dim.width);
	  dim.height += d.height + vgap;
	}

	if ((vertScroll != null) && vertScroll.isVisible()) {
	  Dimension d = vertScroll.getPreferredSize();
	  dim.width += d.width + hgap;
	  dim.height = Math.max(d.height, dim.height);
	}

	Insets insets = target.getInsets();
	dim.width += insets.left + insets.right;
	dim.height += insets.top + insets.bottom;

	return dim;
    }

    /**
     * Lays out the specified container. This method will actually reshape the
     * components in the specified target container in order to satisfy the 
     * constraints of the RadarPaneLayout object. 
     * @param target the component being laid out
     * @see Container
     */
  public void layoutContainer(Container target) {
    debug("layoutContainer");
    Insets insets = target.getInsets();
    Dimension tSize = target.size();
    int top = insets.top;
    int bottom = tSize.height - insets.bottom;
    int left = insets.left;
    int right = tSize.width - insets.right;
	
    if (((RadarPane)target).getScrollbarDisplayPolicy() != RadarPane.SCROLLBARS_NEVER) {
      debug("layoutContainer scrollbarDisplayPolicy="+((RadarPane)target).getScrollbarDisplayPolicy());
        Dimension clientSize = clientPanel.getComponent().size();
        Dimension viewPortSize = new Dimension(right - left,
                                               bottom - top);
        
      if (((RadarPane)target).getScrollbarDisplayPolicy() == ScrollPane.SCROLLBARS_AS_NEEDED) {
        boolean vsbVisible = false;
        boolean hsbVisible = false;
        
	if (clientSize.width > viewPortSize.width) 
          hsbVisible = true;
	
	if (clientSize.height > viewPortSize.height) 
          vsbVisible = true;
	
	if ((!hsbVisible) && (vsbVisible) && 
	    (clientSize.width > (viewPortSize.width - 
	                         vertScroll.getPreferredSize().width))) 
          hsbVisible = true;
	else if ((!vsbVisible) && (hsbVisible) && 
	           (clientSize.height > (viewPortSize.height - 
	                                 horizScroll.getPreferredSize().height))) 
             vsbVisible = true;
           

	if (horizScroll != null)
	  horizScroll.setVisible(hsbVisible);
 	if (vertScroll != null)
	  vertScroll.setVisible(vsbVisible);
  	if (radarToggle != null)
	  radarToggle.setVisible(hsbVisible || vsbVisible);  
       }
      debug("Visible: hsb="+  horizScroll.isVisible()+
         " vsb="+vertScroll.isVisible()+
         " radarToggle="+radarToggle );

      if ((horizScroll != null) && horizScroll.isVisible()) {
	    Dimension hSize = horizScroll.size();
	    horizScroll.setSize(right - left, hSize.height);

	    int vertWidth = 0;
	    if (vertScroll != null) {
	      if(vertScroll.isVisible() || radarToggle.isVisible())
		vertWidth = vertScroll.getPreferredSize().width;
	    } else if ((radarToggle != null) && radarToggle.isVisible()) 
	      vertWidth =  radarToggle.getPreferredSize().width;
	    
	    Dimension d = horizScroll.getPreferredSize();
	    horizScroll.setBounds(left, bottom - d.height, 
				   right - left - (vertWidth + hgap), 
				   d.height);
	    bottom -= d.height + vgap;
      }


      if ((vertScroll != null) && vertScroll.isVisible()) {
	  Dimension vSize = vertScroll.size();
	  vertScroll.setSize(vSize.width, bottom - top);
	  Dimension d = vertScroll.getPreferredSize();
	  int horizHeight = 0;
	  if (horizScroll != null) {
	    // remember if horizScroll.isVisible() == true, then bottom 
	    // already includes the horizScroll height.
	    if (!horizScroll.isVisible() && radarToggle.isVisible())
	      horizHeight =  horizScroll.getPreferredSize().height;
	  } else  if ((radarToggle != null) && radarToggle.isVisible()) {
	      horizHeight =  radarToggle.getPreferredSize().height;
	  }

	  vertScroll.setBounds(right - d.width, top, d.width, bottom - top - horizHeight);
	  
	  right -= d.width + hgap;
      }
	
      if ((radarToggle != null) && radarToggle.isVisible()) 
	  if (horizScroll.isVisible() && vertScroll.isVisible()) 
		radarToggle.setBounds(right+hgap, bottom+vgap,
			tSize.width-right-2*hgap, 
			tSize.height-bottom-2*vgap);
	  else if (horizScroll.isVisible()) {
	  	int rtX = left + horizScroll.getSize().width + hgap;
		radarToggle.setBounds(rtX+hgap, bottom+vgap,
			tSize.width-rtX-2*hgap, 
			tSize.height-bottom-2*vgap); 
	  } else if (vertScroll.isVisible()) {
	    int rtY = top + vertScroll.getSize().height +vgap;
	    radarToggle.setBounds(right+hgap, rtY+vgap,
			tSize.width-right-2*hgap, 
			tSize.height-rtY-2*vgap);
      }
    } // end if (scrollBarDisplayPolicy != SCROLLBARS_NEVER)
    
    if ((clientPanel != null) && clientPanel.isVisible()) {
	    clientPanel.setBounds(left, top, right - left, bottom - top);

	    if (clientPanel.getComponent() != null)
	      checkClientPosition(clientPanel.getComponent().size(),
				clientPanel.size(), target);
    }
    target.repaint();
  }
    
  private void checkClientPosition(Dimension clientSize, Dimension visible, 
				   Component target) {
    debug("checkClientPosition: clientSize="+clientSize+" visible="+visible);
    if ((visible.width <= 0) || (visible.height <= 0))
      return;
    if ((clientSize.width <= 0) || (clientSize.height <= 0)) {
      return;
    }

    // check for blank area resulting from a resize
    Rectangle clientBounds = clientPanel.getComponent().getBounds();
    int newX = clientBounds.x;
    int newY = clientBounds.y;
 
    if (visible.width >= clientBounds.width)
      newX = 0;
    else
      if  ((horizScroll != null) &&(horizScroll.isVisible()))
	if (((-clientBounds.x)+visible.width)>clientBounds.width)
	  newX =  -(clientBounds.width-visible.width);

    if (visible.height >= clientBounds.height)
      newY = 0;
    else
      if ((vertScroll != null) && (vertScroll.isVisible()))
	if (((-clientBounds.y)+visible.height)>clientBounds.height)
	  newY =  -(clientBounds.height-visible.height);

    /*
      JDK 1.1 beta 3 changes the scrollbar model slightly.  You cannot 
      set the value of the scrollbar to greater than the max-visible.
      Previously, the value could range from 0 to max.  
      This imacts this widget since the max was calcualted to be
      clientWidth-visible.  
     */

      if (horizScroll != null)
	horizScroll.setValues(-newX,
			      visible.width,
			      0,
			      clientSize.width);
                // clientSize.width-visible.width); //JDK1.1 beta2-
      if (vertScroll != null)
	vertScroll.setValues(-newY,
			     visible.height,
			     0,
			     clientSize.height);
                // clientSize.height-visible.height); //JDK1.1 beta2-
    
    clientPanel.getComponent(0).setLocation(newX, newY);
    debug("checkClientPosition cPan.getComp.setLoc("+newX+", "+newY+") ");
 }

  /**
   * Returns the String representation of this BorderLayout's values.
   */
  public String toString() {
    return getClass().getName() + "[hgap=" + hgap + ",vgap=" + vgap + "]";
  }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println("RadarPaneLayout: "+message);
  }
}
