/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package vt.cs.widgets;

import java.awt.*;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.awt.image.ImageObserver;
import java.awt.image.ImageProducer;
import java.awt.image.ColorModel;
import java.awt.event.*;
import java.io.Serializable;

/**
 * RadarPane - This is a scrollable panel, which extends <a
href="http://www.javasoft.com/products/JDK/1.1/docs/api/java.awt.ScrollPane.html">java.awt.ScrollPane</a>. 
 *  RadarPane includes a radar
 *  view of the embedded component, which is a scaled down image of 
 *  the entire embedded component.  Users may navigate the viewport
 *  by using the scroll bars or by using the radar view window.
 *  The figure below shows RadarPane.  The scrollable portion is on the
 *  left and the radar view on the right.
</p>
RadarPane is available for download at 
<a href="http://simon.cs.vt.edu/~begolej/download/RadarPane.jar">
http://simon.cs.vt.edu/~begolej/download/RadarPane.jar</a>
</p>
<img src="http://simon.cs.vt.edu/~begolej/images/RadarPane.gif">
</p>
The above demonstration applet with source code may be found at
<a href="http://simon.cs.vt.edu/~begolej/applets/RadarPaneTest/">
http://simon.cs.vt.edu/~begolej/applets/RadarPaneTest/</a></p>
 *
 *  The radar view window may be hidden and displayed by clicking 
 *  the checkbox in the lower right of the scroll pane (See figure above).
 *  The radar view component may be detached from the radar view window.
 *  If the radar view is detached, no radar view window or 
 *  checkbox will appear.</p>

<P>Hits : <IMG SRC="http://www.bev.net/cgi-bin/counter.xbm?RadarPaneS" >
</P>
 *
 * @version 	$Revision: 1.0 $
 * @author 	James Begole (begolej@vt.edu)
 * @author	<a href="mailto:begolej@vt.edu>begolej@vt.edu</a>
 */


public class RadarPaneS extends java.awt.ScrollPane {
	/*
	Although RadarPane extends ScrollPane, it completely overrides 
	the ScrollPane class.  It extends ScrollPane, only so that 
	((RadarPane)radarPaneInstance instanceof ScrollPane) == true) .
	That way, a RadarPane may replace an instance of a ScrollPane.
	*/


  private static String revisionString = "$Revision: 1.0 $";
  private void copyright() {
    System.out.println(getClass().getName()+": "+revisionString+"\n"+
	    "  (c) copyright 1997, James Begole, All rights reseverd.\n"+
	    "  Permission is granted to use this class unmodified.");
  }
  

  /**
   * Specifies that horizontal/vertical scrollbar should be shown 
   * only when the size of the child exceeds the size of the scrollpane
   * in the horizontal/vertical dimension.
   *
   * Note that when both scrollbars are not shown, the radar view
   * window is hidden and the radar view checkbox is also not shown.
   * If a developer wants the radar view to display when both scrollbars
   * are not shown, s/he should detach the radar view and embed
   * it in another AWT container.
   */
  public static final int SCROLLBARS_AS_NEEDED = java.awt.ScrollPane.SCROLLBARS_AS_NEEDED;

  /**
   * Specifies that horizontal/vertical scrollbars should always be
   * shown regardless of the respective sizes of the scrollpane and child.
   */
  public static final int SCROLLBARS_ALWAYS = java.awt.ScrollPane.SCROLLBARS_ALWAYS;

  /**
   * Specifies that horizontal/vertical scrollbars should never be shown
   * regardless of the respective sizes of the scrollpane and child.
   * The radar view checkbox will not be displayed.
   *
   * This mode is more useful here in RadarPane than in ScrollPane, 
   * since users may still navigate the viewport using the radar view.
   */
  public static final int SCROLLBARS_NEVER = java.awt.ScrollPane.SCROLLBARS_NEVER;
	
	/**
	The peer object will have all the functionality.  
	RadarPane is just a wrapper to override the ScrollPane
	methods.
	*/
	private RadarPanePeer peer = null;
	
	public RadarPaneS() {
		super(ScrollPane.SCROLLBARS_NEVER);  
		init(SCROLLBARS_AS_NEEDED);
	}
	
	public RadarPaneS(int scrollbarDisplayPolicy) {
		super(ScrollPane.SCROLLBARS_NEVER);
		init(scrollbarDisplayPolicy);
	}
	
	private void init(int scrollbarDisplayPolicy) {
		super.hide();
		super.disable();
		peer = new RadarPanePeer(scrollbarDisplayPolicy);
		peer.parent = this;
	}
	
	
  /**
   * Adds the specified component to this container at the specified
   * index.  Also notifies the layout manager to add the component to
   * this container's layout using the specified constraints object.
   * <p>
   * This is overriden for RadarPane so that components are added only
   * to the viewport region of the RadarPane.
   * <p>
   * This is the method to override if you want to track every add
   * request to a container.  An overriding method should usually
   * include a call to super.addImpl(comp, constraints, index).
   * @param comp the component to be added
   * @param constraints an object expressing layout contraints for this
   * component
   * @param index the position in the container's list at which to
   * insert the component.  -1 means insert at the end.
   * @see #remove
   * @see LayoutManager
   */
  protected void addImpl(Component comp, Object constraints, int index) {
  	if (peer != null)
  		peer.addImpl(comp, constraints, index);
  }	
  
  /** 
   * Adds the specified component to this scroll pane container.
   * If the scroll pane has an existing child component, that
   * component is removed and the new one is added.  
   * @param comp the component to be added 
   */
  public Component add(Component comp) {
    /*
     *  This method is overriden only so that the above
     *  comments will be displayed in the javadoc documentation
     */
    debug("add( "+comp+" )");
    if (peer != null)
	    return peer.add(comp);
    else
    	return null;
  }

  
   /**
   *  Gets the radar view Component.
   *  The radar view component gives a scaled down 
   *  full view of the client component of this RadarPane
   *  and provides a navigation mechanism in addition to the
   *  scroll bars of this RadarPane.
   *
   *  Use this, if you want the radar view window and the 
   *  radar view checkbox to no longer exist.  
   *  Otherwise, use getRadarView() .
   *
   * @see #getRadarView
   * @return the radar view Component
   */
  public Component detachRadarView() {
	if (peer != null)
		return peer.detachRadarView();
	else
		return null;
  }
  
  /**
   *  Gets the radar view Component.
   *  The radar view component gives a scaled down 
   *  full view of the client component of this RadarPane
   *  and provides a navigation mechanism in addition to the
   *  scroll bars of this RadarPane.
   *
   *  Use this, if you want the radar view window and the radar
   *  radar view checkbox to continue to exist.  
   *  Otherwise, use detachRadarView() .
   *
   * @see #detachRadarView
   * @return the radar view Component
   */
  public Component getRadarView() {
	if (peer != null)
		return peer.getRadarView();
	else
		return null;
  }
  
  /**
   *  Displays the radar view window and the
   *  radar view checkbox. 
   *
   * @see #detachRadarView
   */
  public void attachRadarView() {
	if (peer != null)
		 peer.attachRadarView();
  }
  
  /**
   * Returns the display policy for the scrollbars.
   * @return the display policy for the scrollbars
   */
  public int getScrollbarDisplayPolicy() {
	if (peer != null)
		return peer.getScrollbarDisplayPolicy();
	else
		return SCROLLBARS_AS_NEEDED;
  }

  /**
   * Returns the current size of the scroll pane's view port.
   * @return the size of the view port in pixels
   */
  public Dimension getViewportSize() {
	if (peer != null)
		 return peer.getViewportSize();
	else
		return null;
    }
    
  /**
   * Returns the height that would be occupied by a horizontal
   * scrollbar, which is independent of whether it is currently
   * displayed by the scroll pane or not.
   * @return the height of a horizontal scrollbar in pixels
   */
  public int getHScrollbarHeight() {    
	if (peer != null)
		 return peer.getHScrollbarHeight();
	else
		return 0;
  }
  

  /**
   * Returns the width that would be occupied by a vertical
   * scrollbar, which is independent of whether it is currently
   * displayed by the scroll pane or not.
   * @return the width of a vertical scrollbar in pixels
   */
  public int getVScrollbarWidth() {  
	if (peer != null)
		 return peer.getVScrollbarWidth();
	else
		return 0;
  }
  

  /**
   * Returns the Adjustable object which represents the state of
   * the vertical scrollbar. If the scrollbar display policy is "never",
   * this method returns null.
   */
  public Adjustable getVAdjustable() {
	if (peer != null)
		 return peer.getVAdjustable();
	else
		return null;
  }

  /**
   * Returns the Adjustable object which represents the state of
   * the horizontal scrollbar.  If the scrollbar display policy is "never",
   * this method returns null.
   */
  public Adjustable getHAdjustable() {
	if (peer != null)
		 return peer.getHAdjustable();
	else
		return null;
  }  

	
  /**
   * Scrolls to the specified position within the child component.
   * A call to this method is only valid if the scroll pane contains
   * a child and the specified position is within legal scrolling bounds
   * of the child.  Legal bounds are defined to be the rectangle: 
   * x = 0, y = 0, width = (child width - view port width),
   * height = (child height - view port height).
   * This is a convenience method which interfaces with the Adjustable
   * objects which respresent the state of the scrollbars.
   * @param x the x position to scroll to
   * @param y the y position to scroll to
   * @exception IllegalArgumentException if specified coordinates are
   * not within the legal scrolling bounds of the child component.
   */
  public void setScrollPosition(int x, int y) {
	if (peer != null)
		 peer.setScrollPosition(x, y);
  }  
 
  /**
   * Scrolls to the specified position within the child component.
   * A call to this method is only valid if the scroll pane contains
   * a child and the specified position is within legal scrolling bounds
   * of the child.  Legal bounds are defined to be the rectangle: 
   * x = 0, y = 0, width = (child width - view port width),
   * height = (child height - view port height).
   * This is a convenience method which interfaces with the Adjustable
   * objects which respresent the state of the scrollbars.
   * @param p the Point representing the position to scroll to
   * @exception IllegalArgumentException if specified coordinates are
   * not within the legal scrolling bounds of the child component.
   */
  public void setScrollPosition(Point p) {
    setScrollPosition(p.x, p.y);
  }
  
  /**
   * Returns the current x,y position within the child which is displayed 
   * at the 0,0 location of the scrolled panel's view port.
   * This is a convenience method which interfaces with the adjustable
   * objects which respresent the state of the scrollbars.
   * @return the coordinate position for the current scroll position
   */
  public Point getScrollPosition() {
	if (peer != null)
		 return peer.getScrollPosition();
	else
		return null;
  }

  
  /**
    Hides the radar view window.
   */
  public void hideViewWindow() {
	if (peer != null)
		 peer.hideViewWindow();
  }


/*
 ********** java.awt.Container methods **********
*/
    /** 
     * Returns the number of components in this panel.
     * @see #getComponent
     */
    public int getComponentCount() {
	if (peer != null)
		 return peer.getComponentCount();
	else
		return 0;
  }

    /** 
     * DEPRECATED:  Replaced by getComponentCount().
     */
    public int countComponents() {
	if (peer != null)
		 return peer.countComponents();
	else
		return 0;
    }

    /** 
     * Gets the nth component in this container.
     * @param n the number of the component to get
     * @exception ArrayIndexOutOfBoundsException If the nth value does not 
     * exist.
     */
    public Component getComponent(int n) {
	if (peer != null)
		 return peer.getComponent(n);
	else
		return null;
    }
    
    /**
     * Gets all the components in this container.
     */
    public Component[] getComponents() {
	if (peer != null)
		 return peer.getComponents();
	else
		return null;
    }    

    /**
     * Returns the insets of the container. The insets indicate the size of
     * the border of the container. A Frame, for example, will have a top inset
     * that corresponds to the height of the Frame's title bar. 
     * @see LayoutManager
     */
    public Insets getInsets() {
	if (peer != null)
		 return peer.getInsets();
	else
		return null;
    }

    /**
     * DEPRECATED:  Replaced by getInsets().
     */
    public Insets insets() {
	if (peer != null)
		 return peer.insets();
	else
		return null;
    }


    /**
     * DEPRECATED:  Replaced by add(Component, Object).
     */
    public Component add(String name, Component comp) {
	if (peer != null)
		return peer.add(name, comp);
	else
		return null;
    }

    /** 
     * Adds the specified component to this container at the given
     * position in the container's component list.
     * @param comp the component to be added 
     * @param index the position in the container's list at which to
     * insert the component.  -1 means insert at the end.
     * @see #remove
     */
    public Component add(Component comp, int index) {
	if (peer != null)
		 return peer.add(comp, index);
	else
		return null;
    }
    /**
     * Adds the specified component to this container at the specified
     * index.  Also notifies the layout manager to add the component to
     * the this container's layout using the specified constraints object.
     * @param comp the component to be added
     * @param constraints an object expressing layout contraints for this
     * component
     * @see #remove
     * @see LayoutManager
     */
    public void add(Component comp, Object constraints) {
	if (peer != null)
		 peer.add(comp, constraints);
    }
    /**
     * Adds the specified component to this container with the specified
     * constraints at the specified index.  Also notifies the layout 
     * manager to add the component to the this container's layout using 
     * the specified constraints object.
     * @param comp the component to be added
     * @param constraints an object expressing layout contraints for this
     * @param index the position in the container's list at which to insert
     * the component. -1 means insert at the end.
     * component
     * @see #remove
     * @see LayoutManager
     */
    public void add(Component comp, Object constraints, int index) {
	if (peer != null)
		 peer.add(comp, constraints, index);
    }

    /** 
     * Removes the component at the specified index from this container.
     * @param index the index of the component to be removed
     * @see #add
     */
    public void remove(int index) {
	if (peer != null)
		 peer.remove(index);
    }

    /** 
     * Removes the specified component from this container.
     * @param comp the component to be removed
     * @see #add
     */
    public void remove(Component comp) {
	if (peer != null)
		 peer.remove(comp);
    }
    
    /** 
     * Removes all the components from this container.
     * @see #add
     * @see #remove
     */
    public void removeAll() {
	if (peer != null)
		  peer.removeAll();
    }
    /** 
     * Gets the layout manager for this container.  
     * @see #doLayout
     * @see #setLayout
     */
    public LayoutManager getLayout() {
	if (peer != null)
		  return peer.getLayout();
	else
		return null;
    }
    
    /** 
     * Paints the components in this container.
     * @param g the specified Graphics window
     * @see Component#paint
     * @see Component#paintAll
     */
    public void paintComponents(Graphics g) {
	if (peer != null)
		   peer.paintComponents(g);
    }

    /** 
     * Prints the components in this container.
     * @param g the specified Graphics window
     * @see Component#print
     * @see Component#printAll
     */
    public void printComponents(Graphics g) {
	if (peer != null)
		   peer.printComponents(g);
    }



    /**
     * Adds the specified container listener to receive container events
     * from this container.
     * @param l the container listener
     */ 
    public void addContainerListener(ContainerListener l) {
	if (peer != null)
		   peer.addContainerListener(l);
    }

    /**
     * Removes the specified container listener so it no longer receives
     * container events from this container.
     * @param l the container listener
     */ 
    public void removeContainerListener(ContainerListener l) {
	if (peer != null)
		   peer.removeContainerListener(l);
    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    // REMIND: remove when filtering is done at lower level
//    boolean eventEnabled(AWTEvent e) {
//	if (peer != null)
//		   return peer.eventEnabled(e);
//    }          


// This is protected.  Can't call this method on RadarPanePeer.
// If needed, use a central method in the peer to dispatch these
//    /** 
//     * Processes container events occurring on this container by
//     * dispatching them to any registered ContainerListener objects.
//     * NOTE: This method will not be called unless container events
//     * are enabled for this component; this happens when one of the
//     * following occurs:
//     * a) A ContainerListener object is registered via addContainerListener()
//     * b) Container events are enabled via enableEvents()
//     * @see Component#enableEvents
//     * @param e the container event
//     */  
//    protected void processContainerEvent(ContainerEvent e) {
//	if (peer != null)
//		   return peer.processContainerEvent(e);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    /**
//     * Dispatches an event to this component or one of its sub components.
//     *
//     * @param e the event
//     */
//    void dispatchEvent(AWTEvent e) {
//	if (peer != null)
//		   return peer.dispatchEvent(e);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    /**
//     * Fetchs the top-most (deepest) lightweight component that is interested
//     * in receiving mouse events.
//     */
//    Component getMouseEventTarget(int x, int y) {
//	if (peer != null)
//		   return peer.getMouseEventTarget(x, y);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    /**
//     * This is called by lightweight components that want the containing
//     * windowed parent to enable some kind of events on their behalf.
//     * This is needed for events that are normally only dispatched to 
//     * windows to be accepted so that they can be forwarded downward to 
//     * the lightweight component that has enabled them.
//     */
//    void proxyEnableEvents(long events) {
//		if (peer != null)
//		    peer.proxyEnableEvents(events);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    /**
//     * This is called by lightweight components that have requested focus.
//     * The focus request is propagated upward until a native container is
//     * found, at which point the native container requests focus and records
//     * the component the host is requesting focus for.
//     */
//    void proxyRequestFocus(Component c) {
//		if (peer != null)
//		    peer.proxyRequestFocus(c);
//    }

    /**
     * Checks if the component is contained in the component hierarchy of
     * this container.
     * @param c the component
     */
    public boolean isAncestorOf(Component c) {
	if (peer != null)
		return peer.isAncestorOf(c);
	else 
		return false;
    }

    /**
     * Returns the parameter String of this Container.
     */
    public String paramString() {
	if (peer != null)
		return peer.paramString();
	else 
		return getClass().getName();
    }

    
// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    void setFocusOwner(Component c) {
//	if (peer != null)
//		 peer.setFocusOwner(c);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    void postProcessKeyEvent(KeyEvent e) {
//	if (peer != null)
//		 peer.postProcessKeyEvent(e);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    void transferFocus(Component base) {
//	if (peer != null)
//		 peer.transferFocus(base);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    /**
//     * DEPRECATED:  Replaced by transferFocus(Component).
//     */
//    void nextFocus(Component base) {
//	if (peer != null)
//		 peer.nextFocus(base);
//    }

/*
******** Component methods **************
*/


    /**
     * Checks if this Component is valid. Components are invalidated when
     * they are first shown on the screen.
     * @see #validate
     * @see #invalidate
     */
    public boolean isValid() {
	return (peer != null) && peer.isValid();
    }

    /**
     * Checks if this Component is visible. Components are initially visible 
     * (with the exception of top level components such as Frame).
     * @see #setVisible
     */
    public boolean isVisible() {
	return (peer != null) && peer.isVisible();
    }

    /**
     * Checks if this Component is showing on screen. This means that the 
     * component must be visible, and it must be in a container that is 
     * visible and showing.
     * @see #setVisible
     */
    public boolean isShowing() {
	return (peer != null) && peer.isShowing();
    }

    /**
     * Checks if this Component is enabled. Components are initially enabled.
     * @see #setEnabled
     */
    public boolean isEnabled() {
	return (peer != null) && peer.isEnabled();
    }

    /**
     * Enables a component.
     * @see #isEnabled
     */
    public void setEnabled(boolean b) {
	if (peer != null)
		peer.setEnabled(b);
    }

    /**
     * DEPRECATED:  Replaced by setEnabled(boolean).
     */
    public void enable() {
	if (peer != null)
		peer.enable();
    }


//********
    /**
     * DEPRECATED:  Replaced by setEnabled(boolean).
     */
    public void enable(boolean b) {
	if (peer != null)
		peer.enable(b);
    }

    /**
     * DEPRECATED:  Replaced by setEnabled(boolean).
     */
    public void disable() {
	if (peer != null)
		peer.disable();
    }

    /**
     * Shows or hides the component depending on the boolean flag b.
     * @param b  if true, show the component; otherwise, hide the component.
     * @see #isVisible
     */
    public void setVisible(boolean b) {
	if (peer != null)
		peer.setVisible(b);
    }

    /**
     * DEPRECATED:  Replaced by setVisible(boolean).
     */
    public void show() {
	if (peer != null)
		peer.show();
    }

    /**
     * DEPRECATED:  Replaced by setVisible(boolean).
     */
    public void show(boolean b) {
	if (peer != null)
		peer.show(b);
    }

    /**
     * DEPRECATED:  Replaced by setVisible(boolean).
     */
    public void hide() {
	if (peer != null)
		peer.hide();
    }

    /**
     * Gets the foreground color. If the component does
     * not have a foreground color, the foreground color
     * of its parent is returned.
     * @see #setForeground
     */
    public Color getForeground() {
	if (peer != null)
		return peer.getForeground();
	else
		return null;
    }

    /** 
     * Sets the foreground color.
     * @param c the Color
     * @see #getForeground
     */
    public void setForeground(Color c) {
	if (peer != null)
		peer.setForeground(c);
    }

    /**
     * Gets the background color. If the component does
     * not have a background color, the background color
     * of its parent is returned.
     * @see #setBackground
     */
    public Color getBackground() {
	if (peer != null)
		return peer.getBackground();
	else
		return null;
    }

    /** 
     * Sets the background color.
     * @param c the Color
     * @see #getBackground
     */
    public void setBackground(Color c) {
	if (peer != null)
		peer.setBackground(c);
    }

    /**
     * Gets the font of the component. If the component does
     * not have a font, the font of its parent is returned.
     * @see #setFont
     */
    public Font getFont() {
 	if (peer != null)
		return peer.getFont();
	else
		return null;
    }

    /** 
     * Sets the font of the component.
     * @param f the font
     * @see #getFont
     */
    public synchronized void setFont(Font f) {
	if (peer != null)
		peer.setFont(f);
    }

    /**
     * Gets the locale of the component. If the component does
     * not have a locale, the locale of its parent is returned.
     * @see #setLocale
     * @exception IllegalComponentStateException If the Component 
     * does not have its own locale and has not yet been added to
     * a containment hierarchy such that the locale can be determined
     * from the containing parent.
     */
  public java.util.Locale getLocale() {
 	if (peer != null)
		return peer.getLocale();
	else
		return null;
    }

    /** 
     * Sets the locale of the component.
     * @param l the locale
     * @see #getLocale
     */
    public void setLocale(java.util.Locale l) {
	if (peer != null)
		peer.setLocale(l);
    }

    /**
     * Gets the ColorModel used to display the component on the output device.
     * @see ColorModel
     */
    public java.awt.image.ColorModel getColorModel() {
 	if (peer != null)
		return peer.getColorModel();
	else
		return null;
    }

    /** 
     * Returns the current location of this component.
     * The location will be in the parent's coordinate space.
     * @see #setLocation
     */
    public Point getLocation() {
 	if (peer != null)
		return peer.getLocation();
	else
		return null;
    }

    /** 
     * Returns the current location of this component in the screen's 
     * coordinate space.
     * @see #setLocation
     * @see #getLocation
     */
    public Point getLocationOnScreen() {
 	if (peer != null)
		return peer.getLocationOnScreen();
	else
		return null;
    }


    /** 
     * DEPRECATED:  Replaced by getLocation().
     */
    public Point location() {
	if (peer != null)
		return peer.location();
	else
		return null;
    }

    /** 
     * Moves the Component to a new location. The x and y coordinates
     * are in the parent's coordinate space.
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #getLocation
     * @see #setBounds
     */
    public void setLocation(int x, int y) {
	if (peer != null)
		peer.setLocation(x, y);
    }

    /** 
     * DEPRECATED:  Replaced by setLocation(int, int).
     */
    public void move(int x, int y) {
	if (peer != null)
		peer.move(x, y);
    }

    /** 
     * Moves the Component to a new location. The point p is given in
     * the parent's coordinate space.
     * @param p the new location for the coordinate
     * @see #getLocation
     * @see #setBounds
     */
    public void setLocation(Point p) {
	if (peer != null)
		peer.setLocation(p);
    }

    /** 
     * Returns the current size of this component.
     * @see #setSize
     */
    public Dimension getSize() {
 	if (peer != null)
		return peer.getSize();
	else
		return null;
    }

    /** 
     * DEPRECATED:  Replaced by getSize().
     */
    public Dimension size() {
 	if (peer != null)
		return peer.size();
	else
		return null;
    }

    /**
     * Resizes the Component to the specified width and height.
     * @param width the width of the component
     * @param height the height of the component
     * @see #size
     * @see #setBounds
     */
    public void setSize(int width, int height) {
	 if (peer != null)
		peer.setSize(width, height);
    }

    /**
     * DEPRECATED:  Replaced by setSize(int, int).
     */
    public void resize(int width, int height) {
	 if (peer != null)
		peer.resize(width, height);
    }

    /** 
     * Resizes the Component to the specified dimension.
     * @param d the component dimension
     * @see #size
     * @see #setBounds
     */
    public void setSize(Dimension d) {
	 if (peer != null)
		peer.setSize(d);
    }

    /** 
     * DEPRECATED:  Replaced by setSize(Dimension).
     */
    public void resize(Dimension d) {
	 if (peer != null)
		peer.resize(d);
    }

    /** 
     * Returns the current bounds of this component.
     * @see #setBounds
     */
    public Rectangle getBounds() {
 	if (peer != null)
		return peer.getBounds();
	else
		return null;
    }

    /** 
     * DEPRECATED:  Replaced by getBounds().
     */
    public Rectangle bounds() {
 	if (peer != null)
		return peer.bounds();
	else
		return null;
    }

    /** 
     * Reshapes the Component to the specified bounding box.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the component
     * @param height the height of the component
     * @see #getBounds
     * @see #setLocation
     * @see #setSize
     */
    public void setBounds(int x, int y, int width, int height) {
 	if (peer != null)
		peer.setBounds(x, y, width, height);
    }

    /** 
     * DEPRECATED:  Replaced by setBounds(int, int, int, int).
     */
    public void reshape(int x, int y, int width, int height) {
 	if (peer != null)
		peer.reshape(x, y, width, height);
    }

    /** 
     * Reshapes the Component to the specified bounding box.
     * @param r the new bounding rectangle for the component
     * @see #getBounds
     * @see #setLocation
     * @see #setSize
     */
    public void setBounds(Rectangle r) {
 	if (peer != null)
		peer.setBounds(r);
    }

    
    /** 
     * Returns the preferred size of this container.  
     * @see #getMinimumSize
     */
    public Dimension getPreferredSize() {
	return preferredSize();
    }

    /** 
     * DEPRECATED:  Replaced by getPreferredSize().
     */
    public Dimension preferredSize() {
	if (peer != null)
		  return peer.getPreferredSize();
	else
		return null;
    }

    /** 
     * Returns the minimum size of this container.  
     * @see #getPreferredSize
     */
    public Dimension getMinimumSize() {
	return minimumSize();
    }

    /** 
     * DEPRECATED:  Replaced by getMinimumSize().
     */
    public Dimension minimumSize() {
	if (peer != null)
		  return peer.getMinimumSize();
	else
		return null;
    }

    /** 
     * Returns the maximum size of this container.  
     * @see #getPreferredSize
     */
    public Dimension getMaximumSize() {
	if (peer != null)
		  return peer.getMaximumSize();
	else
		return null;
    }

    /**
     * Returns the alignment along the x axis.  This specifies how
     * the component would like to be aligned relative to other 
     * components.  The value should be a number between 0 and 1
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
     */
    public float getAlignmentX() {
	if (peer != null)
		  return peer.getAlignmentX();
	else
		return (float)0.0;
    }
    
    /**
     * Returns the alignment along the y axis.  This specifies how
     * the component would like to be aligned relative to other 
     * components.  The value should be a number between 0 and 1
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
     */
    public float getAlignmentY() {
	if (peer != null)
		  return peer.getAlignmentY();
	else
		return (float)0.0;
    }    

    
    /** 
     * Invalidates the container.  The container and all parents
     * above it are marked as needing to be laid out.  This method can
     * be called often, so it needs to execute quickly.
     * @see #validate
     * @see #layout
     * @see LayoutManager
     */
    public void invalidate() {
	if (peer != null)
		  peer.invalidate();    
    }

    /** 
     * Validates this Container and all of the components contained
     * within it. 
     * @see #validate
     * @see Component#invalidate
     */
    public void validate() {
	if (peer != null)
		  peer.validate();    
    }

  
    /**
     * Lays out this container by resizing its child to its preferred size.
     * If the new preferred size of the child causes the current scroll
     * position to be invalid, the scroll position is set to the closest
     * valid position.
     *
     * @see Component#validate
     */
    public void doLayout() {
	layout();
    }

    /** 
     * DEPRECATED:  Replaced by doLayout().
     */
    public void layout() {
    	if (peer != null)
    		peer.doLayout();
    }
  

    /**
     * Gets a Graphics context for this component. This method will
     * return null if the component is currently not on the screen.
     * @see #paint
     */
    public Graphics getGraphics() {
	if (peer != null)
		  return peer.getGraphics();
	else
		return null;
    }

    /**
     * Gets the font metrics for this component.
     * @param font the font
     * @see #getFont
     */
    public FontMetrics getFontMetrics(Font font) {
	if (peer != null)
		  return peer.getFontMetrics(font);
	else
		return null;
    }

    /**
     * Set the cursor image to a predefined cursor.
     * @see Cursor
     * @param cursorType one of the cursor constants defined above.
     */
    public synchronized void setCursor(Cursor cursor) {
	if (peer != null)
		  peer.setCursor(cursor);
    }

    /**
     * Gets the cursor set on this component.
     */
    public Cursor getCursor() {
	if (peer != null)
		  return peer.getCursor();
	else
		return null;    
    }

    /** 
     * Paints the container.  This forwards the paint to any lightweight components 
     * that are children of this container.  If this method is reimplemented, 
     * super.paint(g) should be called so that lightweight components are properly
     * rendered.  If a child component is entirely clipped by the current clipping
     * setting in g, paint() will not be forwarded to that child.
     *
     * @param g the specified Graphics window
     * @see #update
     */
    public void paint(Graphics g) {
	if (peer != null)
		   peer.paint(g);
    }
    
    /** 
     * Updates the component. This method is called in
     * response to a call to repaint. You can assume that
     * the background is not cleared.
     * @param g the specified Graphics window
     * @see #paint
     * @see #repaint
     */
    public void update(Graphics g) {
	if (peer != null)
		   peer.update(g);
    }

    /**
     * Paints the component and its subcomponents.
     * @param g the specified Graphics window
     * @see #paint
     */
    public void paintAll(Graphics g) {
	if (peer != null)
		   peer.paintAll(g);
    }
    
    /** 
     * Repaints the component. This will result in a
     * call to update as soon as possible.
     * @see #paint
     */
    public void repaint() {
	if (peer != null)
		   peer.repaint();
    }

    /** 
     * Repaints the component. This will result in a
     * call to update within <em>tm</em> milliseconds.
     * @param tm maximum time in milliseconds before update
     * @see #paint
     */
    public void repaint(long tm) {
	if (peer != null)
		   peer.repaint(tm);
    }

    /** 
     * Repaints part of the component. This will result in a
     * call to update as soon as possible.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width 
     * @param height the height 
     * @see #repaint
     */
    public void repaint(int x, int y, int width, int height) {
	if (peer != null)
		   peer.repaint(0, x, y, width, height);
    }

    /** 
     * Repaints part of the component. This will result in a
     * call to update width <em>tm</em> millseconds.
     * @param tm maximum time in milliseconds before update
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width 
     * @param height the height 
     * @see #repaint
     */
    public void repaint(long tm, int x, int y, int width, int height) {
	if (peer != null)
		   peer.repaint(tm, x, y, width, height);
    }

    /** 
     * Prints the container.  This forwards the print to any lightweight components 
     * that are children of this container.  If this method is reimplemented, 
     * super.print(g) should be called so that lightweight components are properly
     * rendered.  If a child component is entirely clipped by the current clipping
     * setting in g, print() will not be forwarded to that child.
     *
     * @param g the specified Graphics window
     * @see #update
     */
    public void print(Graphics g) {
	if (peer != null)
		   peer.print(g);
    }

    /**
     * Prints the component and its subcomponents.
     * @param g the specified Graphics window
     * @see #print
     */
    public void printAll(Graphics g) {
	if (peer != null)
		   peer.printAll(g);
    }

    /**
     * Repaints the component when the image has changed.
     * @return true if image has changed; false otherwise.
     */
    public boolean imageUpdate(Image img, int flags,
			       int x, int y, int w, int h) {
	if (peer != null)
		   return peer.imageUpdate(img, flags, x, y, w, h);
	else
		return false;
    }

    /**
     * Creates an image from the specified image producer.
     * @param producer the image producer
     */
    public Image createImage(ImageProducer producer) {
	if (peer != null)
		   return peer.createImage(producer);
	else
		return null;
    }

    /**
     * Creates an off-screen drawable Image to be used for double buffering.
     * @param width the specified width
     * @param height the specified height
     */
    public Image createImage(int width, int height) {
	if (peer != null)
		   return peer.createImage(width, height);
	else
		return null;
    }
 
    /**
     * Prepares an image for rendering on this Component.  The image
     * data is downloaded asynchronously in another thread and the
     * appropriate screen representation of the image is generated.
     * @param image the Image to prepare a screen representation for
     * @param observer the ImageObserver object to be notified as the
     *        image is being prepared
     * @return true if the image has already been fully prepared
     * @see ImageObserver
     */
    public boolean prepareImage(Image image, ImageObserver observer) {
	if (peer != null)
		   return peer.prepareImage(image, observer);
	else
		return false;
    }

    /**
     * Prepares an image for rendering on this Component at the
     * specified width and height.  The image data is downloaded
     * asynchronously in another thread and an appropriately scaled
     * screen representation of the image is generated.
     * @param image the Image to prepare a screen representation for
     * @param width the width of the desired screen representation
     * @param height the height of the desired screen representation
     * @param observer the ImageObserver object to be notified as the
     *        image is being prepared
     * @return true if the image has already been fully prepared
     * @see ImageObserver
     */
    public boolean prepareImage(Image image, int width, int height,
				ImageObserver observer) {
	if (peer != null)
		   return peer.prepareImage(image, width, height, observer);
	else
		return false;
    }

    /**
     * Returns the status of the construction of a screen representation
     * of the specified image.
     * This method does not cause the image to begin loading. Use the
     * prepareImage method to force the loading of an image.
     * @param image the Image to check the status of
     * @param observer the ImageObserver object to be notified as the
     *        image is being prepared
     * @return the boolean OR of the ImageObserver flags for the
     *         data that is currently available
     * @see ImageObserver
     * @see #prepareImage
     */
    public int checkImage(Image image, ImageObserver observer) {
	if (peer != null)
		   return peer.checkImage(image, -1, -1, observer);
	else
		return 0;
    }

    /**
     * Returns the status of the construction of a scaled screen
     * representation of the specified image.
     * This method does not cause the image to begin loading, use the
     * prepareImage method to force the loading of an image.
     * @param image the Image to check the status of
     * @param width the width of the scaled version to check the status of
     * @param height the height of the scaled version to check the status of
     * @param observer the ImageObserver object to be notified as the
     *        image is being prepared
     * @return the boolean OR of the ImageObserver flags for the
     *         data that is currently available
     * @see ImageObserver
     * @see #prepareImage
     */
    public int checkImage(Image image, int width, int height,
			  ImageObserver observer) {
	if (peer != null)
		   return peer.checkImage(image, width, height, observer);
	else
		return 0;
    }

    /**  
     * Checks whether this component "contains" the specified (x, y)
     * location, where x and y are defined to be relative to the 
     * coordinate system of this component.  
     * @param x the x coordinate 
     * @param y the y coordinate
     * @see #getComponentAt
     */
    public boolean contains(int x, int y) {
	if (peer != null)
		return peer.contains(x, y);
	else
		return false;
    }

    /**  
     * DEPRECATED:  Replaced by contains(int, int).
     */
    public boolean inside(int x, int y) {
	if (peer != null)
		return peer.inside(x, y);
	else
		return false;
    }

    /**  
     * Checks whether this component "contains" the specified point,
     * where x and y in the point are defined to be relative to the 
     * coordinate system of this component.  
     * @param p the point
     * @see #getComponentAt
     */
    public boolean contains(Point p) {
	if (peer != null)
		return peer.contains(p);
	else
		return false;
    }

    /** 
     * Returns the component or subcomponent that contains the x,y location.
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #contains
     */
    public Component getComponentAt(int x, int y) {
	if (peer != null)
		return peer.getComponentAt(x, y);
	else
		return null;
    }

    /** 
     * DEPRECATED:  Replaced by getComponentAt(int, int).
     */
    public Component locate(int x, int y) {
	if (peer != null)
		return peer.locate(x, y);
	else
		return null;
    }

    /** 
     * Returns the component or subcomponent that contains the
     * specified point.
     * @param p the point
     * @see #contains
     */
    public Component getComponentAt(Point p) {
	if (peer != null)
		return peer.getComponentAt(p);
	else
		return null;
    }

    /**
     * DEPRECATED: replaced by dispatchEvent(AWTEvent e)
     */
    public void deliverEvent(Event e) {
	if (peer != null)
		peer.deliverEvent(e);
    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    /**
//     * Dispatches an event to this component or one of its sub components.
//     * @param e the event
//     */
//    void dispatchEvent(AWTEvent e) {
//	if (peer != null)
//		peer.dispatchEvent(e);
//    }

    /**
     * DEPRECATED:  Replaced by dispatchEvent(AWTEvent e);
     * 
     * Posts an event to this component. This will result in a call
     * to handleEvent. If handleEvent returns false the event is
     * passed on to the parent of this component.
     * @param e the event
     * @see #handleEvent
     * @see #deliverEvent
     */
    public boolean postEvent(Event e) {
	if (peer != null)
		return peer.postEvent(e);
	else
		return false;
    }


    // Event source interfaces

    /**
     * Adds the specified component listener to receive component events
     * from this component.
     * @param l the component listener
     */  
    public synchronized void addComponentListener(ComponentListener l) {
	if (peer != null)
		peer.addComponentListener(l);
    }

    /**
     * Removes the specified listener so it no longer receives component
     * events from this component.
     * @param l the component listener
     */ 
    public synchronized void removeComponentListener(ComponentListener l) {
	if (peer != null)
		peer.removeComponentListener(l);
    }        

    /**
     * Adds the specified focus listener to receive focus events
     * from this component.
     * @param l the focus listener
     */      
    public synchronized void addFocusListener(FocusListener l) {
	if (peer != null)
		peer.addFocusListener(l);
    }

    /**
     * Removes the specified focus listener so it no longer receives focus
     * events from this component.
     * @param l the focus listener
     */ 
    public synchronized void removeFocusListener(FocusListener l) {
	if (peer != null)
		peer.removeFocusListener(l);
    }   

    /**
     * Adds the specified key listener to receive key events
     * from this component.
     * @param l the key listener
     */      
    public synchronized void addKeyListener(KeyListener l) {
	if (peer != null)
		peer.addKeyListener(l);
    }

    /**
     * Removes the specified key listener so it no longer receives key
     * events from this component.
     * @param l the key listener
     */ 
    public synchronized void removeKeyListener(KeyListener l) {
	if (peer != null)
		peer.removeKeyListener(l);
    }  

    /**
     * Adds the specified mouse listener to receive mouse events
     * from this component.
     * @param l the mouse listener
     */      
    public synchronized void addMouseListener(MouseListener l) {
	if (peer != null)
		peer.addMouseListener(l);
    }

    /**
     * Removes the specified mouse listener so it no longer receives mouse
     * events from this component.
     * @param l the mouse listener
     */ 
    public synchronized void removeMouseListener(MouseListener l) {
	if (peer != null)
		peer.removeMouseListener(l);
    }

    /**
     * Adds the specified mouse motion listener to receive mouse motion events
     * from this component.
     * @param l the mouse motion listener
     */  
    public synchronized void addMouseMotionListener(MouseMotionListener l) {
	if (peer != null)
		peer.addMouseMotionListener(l);
    }

    /**
     * Removes the specified mouse motion listener so it no longer 
     * receives mouse motion events from this component.
     * @param l the mouse motion listener
     */ 
    public synchronized void removeMouseMotionListener(MouseMotionListener l) {
	if (peer != null)
		peer.removeMouseMotionListener(l);
    }    


    /** 
     * Processes events occurring on this component.  By default this
     * method will call the appropriate processXXXEvent method for the 
     * class of event.
     * @see #processComponentEvent
     * @see #processFocusEvent
     * @see #processKeyEvent
     * @see #processMouseEvent
     * @see #processMouseMotionEvent
     * @param e the event
     */   
    protected void processEvent(AWTEvent e) {
	if (peer != null)
		peer.processEvent(e);
    }
    
// This is protected.  Can't call this method on RadarPanePeer.
// If needed, use a central method in the peer to dispatch these
//    /** 
//     * Processes component events occurring on this component by
//     * dispatching them to any registered ComponentListener objects.
//     * NOTE: This method will not be called unless component events
//     * are enabled for this component; this happens when one of the
//     * following occurs:
//     * a) A ComponentListener object is registered via addComponentListner()
//     * b) Component events are enabled via enableEvents()
//     * @param e the component event
//     */   
//    protected void processComponentEvent(ComponentEvent e) {
//	if (peer != null)
//		peer.processComponentEvent(e);
//    }

// This is protected.  Can't call this method on RadarPanePeer.
// If needed, use a central method in the peer to dispatch these
//    /** 
//     * Processes focus events occurring on this component by
//     * dispatching them to any registered FocusListener objects.
//     * NOTE: This method will not be called unless focus events
//     * are enabled for this component; this happens when one of the
//     * following occurs:
//     * a) A FocusListener object is registered via addFocusListener()
//     * b) Focus events are enabled via enableEvents()
//     * @param e the focus event
//     */       
//    protected void processFocusEvent(FocusEvent e) {
//	if (peer != null)
//		peer.processFocusEvent(e);
//    }

// This is protected.  Can't call this method on RadarPanePeer.
// If needed, use a central method in the peer to dispatch these
//    /** 
//     * Processes key events occurring on this component by
//     * dispatching them to any registered KeyListener objects.
//     * NOTE: This method will not be called unless key events
//     * are enabled for this component; this happens when one of the
//     * following occurs:
//     * a) A KeyListener object is registered via addKeyListener()
//     * b) Key events are enabled via enableEvents()
//     * @param e the key event
//     */   
//    protected void processKeyEvent(KeyEvent e) {
//	if (peer != null)
//		peer.processKeyEvent(e);
//    }

// This is protected.  Can't call this method on RadarPanePeer.
// If needed, use a central method in the peer to dispatch these
//    /** 
//     * Processes mouse events occurring on this component by
//     * dispatching them to any registered MouseListener objects.
//     * NOTE: This method will not be called unless mouse events
//     * are enabled for this component; this happens when one of the
//     * following occurs:
//     * a) A MouseListener object is registered via addMouseListener()
//     * b) Mouse events are enabled via enableEvents()
//     * @param e the mouse event
//     */   
//    protected void processMouseEvent(MouseEvent e) {
//	if (peer != null)
//		peer.processMouseEvent(e);
//    }

// This is protected.  Can't call this method on RadarPanePeer.
// If needed, use a central method in the peer to dispatch these
//    /** 
//     * Processes mouse motion events occurring on this component by
//     * dispatching them to any registered MouseMotionListener objects.
//     * NOTE: This method will not be called unless mouse motion events
//     * are enabled for this component; this happens when one of the
//     * following occurs:
//     * a) A MouseMotionListener object is registered via addMouseMotionListener()
//     * b) Mouse Motion events are enabled via enableEvents()
//     * @param e the mouse motion event
//     */   
//    protected void processMouseMotionEvent(MouseEvent e) {
//	if (peer != null)
//		peer.processMouseMotionEvent(e);
//    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    boolean postsOldMouseEvents() {
//	if (peer != null)
//		peer.postsOldMouseEvents();
//	else
//		return false;
//    }

    /**
     * DEPRECATED:  Replaced by processEvent(AWTEvent e);
     *
     * Handles the event. Returns true if the event is handled and
     * should not be passed to the parent of this component. The default
     * event handler calls some helper methods to make life easier
     * on the programmer.
     * @param evt the event
     * @see #mouseEnter
     * @see #mouseExit
     * @see #mouseMove
     * @see #mouseDown
     * @see #mouseDrag
     * @see #mouseUp
     * @see #keyDown
     * @see #action
     */
    public boolean handleEvent(Event evt) {
	if (peer != null)
		return peer.handleEvent(evt);
	else
		return false;
    }



    /**
     * DEPRECATED:  Replaced by processMouseEvent(MouseEvent e);
     *
     * Called if the mouse is down.
     * @param evt the event 
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #handleEvent
     */
    public boolean mouseDown(Event evt, int x, int y) {
	if (peer != null)
		return peer.mouseDown(evt, x, y);
	else
		return false;
    }

    /**
     * DEPRECATED:  Replaced by processMouseMotionEvent(MouseEvent e);
     *
     * Called if the mouse is dragged (the mouse button is down).
     * @param evt the event
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #handleEvent
     */
    public boolean mouseDrag(Event evt, int x, int y) {
	if (peer != null)
		return peer.mouseDrag(evt, x, y);
	else
		return false;
     }

    /**
     * DEPRECATED:  Replaced by processMouseEvent(MouseEvent e);
     *
     * Called if the mouse is up.
     * @param evt the event
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #handleEvent
     */
    public boolean mouseUp(Event evt, int x, int y) {
	if (peer != null)
		return peer.mouseUp(evt, x, y);
	else
		return false;
    }

    /**
     * DEPRECATED:  Replaced by processMouseMotionEvent(MouseEvent e);
     *
     * Called if the mouse moves (the mouse button is up).
     * @param evt the event
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #handleEvent
     */
    public boolean mouseMove(Event evt, int x, int y) {
	if (peer != null)
		return peer.mouseMove(evt, x, y);
	else
		return false;
    }

    /**
     * DEPRECATED:  Replaced by processMouseEvent(MouseEvent e);
     *
     * Called when the mouse enters the component.
     * @param evt the event
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #handleEvent
     */
    public boolean mouseEnter(Event evt, int x, int y) {
	if (peer != null)
		return peer.mouseEnter(evt, x, y);
	else
		return false;
    }

    /**
     * DEPRECATED:  Replaced by processMouseEvent(MouseEvent e);
     *
     * Called when the mouse exits the component.
     * @param evt the event
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #handleEvent
     */
    public boolean mouseExit(Event evt, int x, int y) {
	if (peer != null)
		return peer.mouseExit(evt, x, y);
	else
		return false;
    }

    /**
     * DEPRECATED:  Replaced by processKeyEvent(KeyEvent e);
     *
     * Called if a character is pressed.
     * @param evt the event
     * @param key the key that's pressed
     * @see #handleEvent
     */
    public boolean keyDown(Event evt, int key) {
	if (peer != null)
		return peer.keyDown(evt, key);
	else
		return false;
    }

    /**
     * DEPRECATED:  Replaced by processKeyEvent(KeyEvent e);
     *
     * Called if a character is released.
     * @param evt the event
     * @param key the key that's released
     * @see #handleEvent
     */
    public boolean keyUp(Event evt, int key) {
	if (peer != null)
		return peer.keyUp(evt, key);
	else
		return false;
    }

    /**
     * DEPRECATED:  Replaced by processActionEvent(KeyEvent e);
     *
     * Called if an action occurs in the Component.
     * @param evt the event
     * @param what the action that's occuring
     * @see #handleEvent
     */
    public boolean action(Event evt, Object what) {
	if (peer != null)
		return peer.action(evt, what);
	else
		return false;
    }

     /**
     * Creates the scroll pane's peer. 
     */
    public void addNotify() {
	if (peer != null)
		 peer.addNotify();
    }

    /** 
     * Notifies the Component that it has been removed from its
     * container and it needs to destroy the peer.
     * This method should be called by Container.remove, and not by user
     * code directly.
     * @see #addNotify
     */
    public void removeNotify() {
	if (peer != null)
		 peer.removeNotify();
    }


    /** 
     * DEPRECATED:  Replaced by processFocusEvent(FocusEvent e);
     *
     * Indicates that this component has received the input focus.
     * @see #requestFocus
     * @see #lostFocus
     */
    public boolean gotFocus(Event evt, Object what) {
	if (peer != null)
		return peer.gotFocus(evt, what);
	else
		return false;
    }

    /** 
     * DEPRECATED:  Replaced by processFocusEvent(FocusEvent e);
     *
     * Indicates that this component has lost the input focus.  
     * @see #requestFocus
     * @see #gotFocus
     */
    public boolean lostFocus(Event evt, Object what) {
	if (peer != null)
		return peer.lostFocus(evt, what);
	else
		return false;
    }

    /** 
     * Requests the input focus. The gotFocus() method will be called
     * if this method is successful.  The component must be visible
     * on the screen for this request to be granted.
     * @see #gotFocus
     */
    public void requestFocus() {
 	if (peer != null)
		peer.requestFocus();
    }

    /**
     * Transfers the focus to the next component.
     * @see #requestFocus
     * @see #gotFocus
     */
     public void transferFocus() {
 	if (peer != null)
		peer.transferFocus();
     }

    /**
     * DEPRECATED:  Replaced by transferFocus().
     */
     public void nextFocus() {
 	if (peer != null)
		peer.nextFocus();
     }

    /**
     * Adds the specified popup menu to the component.
     * @param popup the popup menu to be added to the component
     */
    public synchronized void add(PopupMenu popup) {
 	if (peer != null)
		peer.add(popup);
    }

    /**
     * Removes the specified popup menu from the component.
     * @param popup the popup menu to be removed
     */
    public synchronized void remove(MenuComponent popup) {
 	if (peer != null)
		peer.remove(popup);
    }

    /**
     * Prints a listing to System.out.
     */
    public void list() {
 	if (peer != null)
		peer.list();
    }

    /**
     * Prints a listing to the specified print stream.
     * @param out the Stream name
     */
    public void list(PrintStream out) {
 	if (peer != null)
		peer.list(out);
    }

    /**
     * Prints out a list, starting at the specified indention, to the specified 
     * print stream.
     * @param out the Stream name
     * @param indent the start of the list 
     */
    public void list(PrintStream out, int indent) {
 	if (peer != null)
		peer.list(out, indent);
    }

    /**
     * Prints a listing to the specified print writer.
     */
    public void list(PrintWriter out) {
 	if (peer != null)
		peer.list(out);
    }

    /**
     * Prints out a list, starting at the specified indention, to the specified 
     * print writer.
     */
    public void list(PrintWriter out, int indent) {
 	if (peer != null)
		peer.list(out, indent);
    }

    /**
     * Returns whether this component can be traversed using
     * Tab or Shift-Tab keyboard focus traversal.
     */
    public boolean isFocusTraversable() {
	if (peer != null)
		return peer.isFocusTraversable();
	else
		return false;
    }

// This is package protected.  Could put RadarPane in java.awt, but don't want to
//    /**
//     * Fetch the native container somewhere higher up in the component
//     * tree that contains this component.  
//     */
//    Container getNativeContainer() {
//	if (peer != null)
//		return peer.getNativeContainer();
//	else
//		return null;
//    }


  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }


}
